<?php /** @noinspection ALL */

namespace Slovocast\Domain\Repository\Channel;

use DateTime;
use DateTimeImmutable;
use Exception;
use Throwable;
use Slovocast\Domain\Repository\Channel\ChannelRepositoryInterface;
use Slovocast\Domain\Entity\Channel;
use Slovocast\Domain\Entity\User;
use Slovocast\Exception\EntityNotFoundException;
use Slovocast\Infrastructure\Api\Database\DatabaseHandlerInterface;

class ChannelRepository implements ChannelRepositoryInterface
{
    public function __construct(
        protected DatabaseHandlerInterface $db
    ) { }

    /**
     * @param array $results Result Set from the Query
     * @return Channel
     * @throws Exception
     */
    protected function createChannelFromResults(array $results): Channel
    {
        return Channel::fromArray([
            'id' => $results['id'],
            'name' => $results['name'],
            'slug' => $results['slug'],
            'description' => $results['description'],
            'link' => $results['link'],
            'language' => $results['language'],
            'copyright' => $results['copyright'],
            'explicit' => $results['explicit'],
            'createdAt' => new DateTimeImmutable($results['created_at']),
            'updatedAt' => new DateTime($results['updated_at'])
        ]);
    }

    /**
     * @param int $id
     * @return Channel
     * @throws EntityNotFoundException
     * @throws Throwable
     */
    public function get(int $id): Channel
    {
        $query = "SELECT * FROM channels WHERE id = :id";
        $statement = $this->db->getConnection()->prepare($query);
        $statement->execute([ ':id' => $id ]);
        $results = $statement->fetch(\PDO::FETCH_ASSOC);

        if (!is_array($results) || !count($results)) {
            throw new EntityNotFoundException("Unable to find Channel");
        }

        return $this->createChannelFromResults($results);
    }

    /**
     * @throws EntityNotFoundException
     * @throws Throwable
     */
    public function getFromUser(User $user): Channel
    {
        $query = "SELECT * FROM channels WHERE owner_id = ?";
        $statement = this->db->getConnection()->prepare($query);
        $statement->execute([ ':id' => $user->getId() ]);
        $results = $statement->fetch(\PDO::FETCH_ASSOC);

        if (!is_array($results) || !count($results)) {
            throw new EntityNotFoundException("Unable to find Channel");
        }

        return $this->createChannelFromResults($results);
    }

    /**
     * @throws EntityNotFoundException
     * @throws Throwable
     */
    public function getFromSlug(string $slug): Channel
    {
        $query = "SELECT * FROM channels WHERE slug = :slug";
        $statement = $this->db->getConnection()->prepare($query);
        $statement->execute([ ':slug' => $slug ]);
        $results = $statement->fetch(\PDO::FETCH_ASSOC);

        if (!is_array($results) || !count($results)) {
            throw new EntityNotFoundException("Unable to find Channel");
        }

        return $this->createChannelFromResults($results);
    }

    /**
     * @throws Throwable
     */
    public function create(Channel $channel, User $owner): Channel
    {
        $query = " 
            INSERT INTO channels (name, slug, description, link,
                                  language, copyright, explicit, owner_id)
            VALUES (?, ?, ?, ?, ?, ?, ?, ?)
        ";

        $statement = $this->db->getConnection()->prepare($query);
        $statement->execute([
            $channel->getName(),
            $channel->getSlug(),
            $channel->getDescription(),
            $channel->getLink(),
            $channel->getLanguage(),
            $channel->getCopyright(),
            $channel->isExplicit(),
            $owner->getId()
        ]);

        $insertId = $this->db->getConnection()->lastInsertId();
        $channel->setId($insertId);
        $channel->setCreatedAt(new DateTimeImmutable());
        $channel->setUpdatedAt(new DateTime());
        return $channel;
    }

    /**
     * There are side-effects to this method, where the given Channel entity
     * will have its `updatedAt` property set if the update in the database
     * is successful.
     *
     * @param Channel $channel
     * @return bool
     * @throws Throwable
     */
    public function update(Channel $channel): bool
    {
        $query = "
            UPDATE channels SET name = ?,
                                slug = ?,
                                description = ?,
                                link = ?,
                                language = ?,
                                copyright = ?,
                                explicit = ?
            WHERE id = ?;
        ";

        $statement = $this->db->getConnection()->prepare($query);
        $result = $statement->execute([
            $channel->getName(),
            $channel->getSlug(),
            $channel->getDescription(),
            $channel->getLink(),
            $channel->getLanguage(),
            $channel->getCopyright(),
            $channel->isExplicit(),
            $channel->getId(),
        ]);

        if ($result == true) {
            $channel->setUpdatedAt(new DateTime());
        }

        return $result;
    }

    /**
     * Don't delete for now.
     */
    public function delete(Channel $channel): bool
    {
        return false;
    }
}
