<?php

namespace Slovocast\Domain\Repository\Image;

use DateTimeImmutable;
use DateTime;
use Slovocast\Exception\EntityNotFoundException;
use Slovocast\Infrastructure\Api\Database\DatabaseHandlerInterface;
use Slovocast\Domain\Repository\Image\ImageRepositoryInterface;
use Slovocast\Domain\Entity\Image;

class ImageRepository implements ImageRepositoryInterface
{
    public function __construct(
        private DatabaseHandlerInterface $db
    ) { }

    /**
     * @param array $results The results of a query
     */
    protected function createImageFromResults(array $results): Image
    {
        $results['createdAt'] = new DateTimeImmutable($results['created_at']);
        $results['updatedAt'] = new DateTime($results['updated_at']);
        return Image::fromArray($results);
    }
   
    public function get(int $id): Image
    {
        $query = "SELECT * FROM images WHERE id = :id";
        $statement = $this->db->getConnection()->prepare($query);
        $statement->execute([ ':id' => $id ]);
        $results = $statement->fetch(\PDO::FETCH_ASSOC);

        if (!is_array($results) || !count($results)) {
            throw new EntityNotFoundException("Unable to find image.");
        }

        return $this->createImageFromResults($results);
    }

    public function getFromUrl(string $url): Image
    {
        $query = "SELECT * FROM images WHERE url = :url";
        $statement = $this->db->getConnection()->prepare($query);
        $statement->execute([ ':url' => $url ]);
        $results = $statement->fetch(\PDO::FETCH_ASSOC);

        if (!is_array($results) || !count($results)) {
            throw new EntityNotFoundException("Unabel to find image.");
        }

        return $this->createImageFromResults($results);
    }

    public function create(Image $image): Image
    {
        $query = "INSERT INTO 
            images (url, title, width, height)
            VALUES (:url, :title, :width, :height)";

        $statement = $this->db->getConnection()->prepare($query); 
        $statement->execute([
            ':url' => $image->getUrl(),
            ':title' => $image->getTitle(),
            ':width' => $image->getWidth(),
            ':height' => $image->getHeight(),
        ]);

        $insertId = $this->db->getConnection()->lastInsertId();
        $image->setId($insertId);
        $image->setCreatedAt(new DateTimeImmutable());
        $image->setUpdatedAt(new DateTime());

        return $image;
    }

    public function update(Image $image): bool
    {
        $query = "UPDATE images 
            SET url = ?,
                title = ?,
                width = ?,
                height = ?
            WHERE id = ?";

        $statement = $this->db->getConnection()->prepare($query);
        $results = $statement->execute([
            $image->getUrl(),
            $image->getTitle(),
            $image->getWidth(),
            $image->getHeight(),
            $image->getId(),
        ]);

        if ($results == true) {
            $image->setUpdatedAt(new DateTime());
        }
        
        return $results;
    }

    public function delete(Image $image): bool
    {
        return false;
    }
}
